import csv
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
from mpl_toolkits.axes_grid1 import make_axes_locatable
import sys,os
import json
import jsonpickle
from scipy.spatial import Voronoi, voronoi_plot_2d
from functions import nums,haversine,calculate_bearing,plotDict,\
    polygon_area,test_all, plotMelt,meltrate,meltrateSalinity,plotVorTemps,myDictImputer
from sklearn.impute import SimpleImputer
from sklearn.linear_model import LinearRegression,Lasso
import datetime as dt
import collections


readFile = False 
smoothApproach = True
#    m_rate = vol*rosw*temp*oc_heatCap*(latentHeat+oc_heatCap*(temp-freezeTemp)+ice_heatCap*(freezeTemp-basalTemp))

#from subprocess import call
#call(["pwd"])
dirMain = os.path.realpath('..')  + "/" 
if dirMain == '/Users/':
    dirMain = "/Users/christiedjidjev/OneDrive/Ice Shelf Project/" 
dirCAT = dirMain + "Data/CAT_files/"
os.system("rm " + "-f " + dirCAT + ".DS_Store")
num_rows=2519086
year_col = 6
month_col = 5
day_col = 4
hour_col = 7
min_col = 8
salin_col = 3
temp_col = 1
cm2m = 0.01 #converting cm to m
sec2min = 60 #converting sec to min
measIntervalInMinutes = 5
#max_column = 5
file_name=np.empty(num_rows,dtype=object)
temp_dict={}
sal_dict={}

f = "Codes/cat2rcm.txt"
rcmFile = {}
catFile = {}
sameFile = {}

#reads cat2rcm correspondence 
with open(dirMain+f) as infile:
    for line in infile:
        row = line.split()
        rcmFile[row[0]] = row[1]
        catFile[row[1]] = row[0]
        sameFile[row[1]] = row[1]

#extracting information from cat files
files2 = {}
if readFile: 
  for f in os.listdir(dirCAT):
    if f not in rcmFile: continue
    print f
    files2[f]=True
    with open(dirCAT+f) as infile:
      line_num=0
      for line in infile:
          line_num= line_num+1
          if line_num ==1:
              name_val = line.split()[2]
              if (name_val == 'amisor8' or name_val == 'amisor9'): break
          if line_num < 14: continue
          line = " ".join(line.split()).replace(' ', '\t')
          row = line.split("\t")
          T=float(row[temp_col]) #Temperature 
          S=float(row[salin_col]) #Salinity
          if T < -990: continue          
          year=int(row[year_col]) #Year 
          month=int(row[month_col]) #Month 
          day=int(row[day_col]) #Day 
          hour=int(row[hour_col]) #Hour 
          if hour == 24: hour = 23
          if year == 2002 and month == 12: year = 2001
          minutes=int(row[min_col]) #Min
          date_str=str(year)+'.'+str(month).zfill(2)+'.'+str(day).zfill(2)+'.'+str(hour)
          #date_str=str(year)+'.'+str(month)+'.'+str(day)+'.'+str(hour)
          temp_dict[date_str]=temp_dict.get(date_str,{})
          temp_dict[date_str][f]=T
          sal_dict[date_str]=sal_dict.get(date_str,{})
          sal_dict[date_str][f]=S
            
if readFile:
    fname = dirMain + "Codes/temp.txt"
    with open(fname, 'w') as f:
        json.dump(temp_dict,f)
    fname = dirMain + "Codes/sal.txt"
    with open(fname, 'w') as f:
        json.dump(sal_dict,f)

if not readFile:
    fname = dirMain + "Codes/temp.txt"
    with open(fname, 'r') as f2:
        temp_dict = json.load(f2)
    fname = dirMain + "Codes/sal.txt"
    with open(fname, 'r') as f2:
        sal_dict = json.load(f2)

#gets the vol data computed in Christie_project.py
fname=dirMain + "Codes/vol.txt"
with open(fname, 'r') as f2:
    volume_dict = json.load(f2)
    
temp_dict = myDictImputer(temp_dict) #3480 values fixed

sumMelt = 0
sumVol = 0
sumTemp = 0
sumSal = 0

meltDict = {}
tempDict = {}
salDict = {}

def  estimate_value(value_dict, time_ind, file_ind, neighb1, neighb2,rename):
    val0 = value_dict[time_ind][file_ind]
    val1 = 0
    count = 0
    for f in neighb1:
        try:    
            val1 += value_dict[time_ind][rename[f]]
            count += 1
        except: continue
    if count > 0: val1 /= count
    val2 = 0
    count = 0
    for f in neighb2:
        try:
            val2 += value_dict[time_ind][rename[f]]
            count += 1
        except: continue
    if count > 0: val2 /= count
    return val0, val1, val2 # values at thee triangle points

#get stored data
files=['regions','point_region','vertices','point_index','neighb']
objects = []
for i,f in enumerate(files):
    fname = dirMain + "Codes/" + f + ".txt"
    with open(fname, 'r') as f2: o = json.load(f2)
    o = jsonpickle.decode(o)
    objects.append(o)
[regions,point_region,vertices,point_index,neighb] = objects

#compute meltrate based on temp

if smoothApproach:
  for time_ind in temp_dict:
    meltSumInstr = 0
    numInstr = 0
    for file_ind in temp_dict[time_ind]:
        p=point_index[rcmFile[file_ind]]
        region = regions[point_region[p]]
        for i in range(len(region)):
            v1 = region[i] 
            if i < len(region)-1: v2 = region[i+1] 
            else: v2 = region[0]
            val0, val1, val2 = estimate_value(temp_dict, time_ind, file_ind, neighb[v1], neighb[v2], catFile)
            T=(val0+val1+val2)/3
            if time_ind not in volume_dict: continue
            if rcmFile[file_ind] not in volume_dict[time_ind]: continue
            #getting the vol for a given time and instrument
            val0, val1, val2 = estimate_value(volume_dict, time_ind, rcmFile[file_ind], neighb[v1], neighb[v2], sameFile)
            V=(val0+val1+val2)/3/len(region)
            melt = meltrate(V,T)
            sumMelt += melt
            meltSumInstr += melt
            numInstr += 1
    if numInstr == 0: continue
    t = dt.datetime.strptime(time_ind,'%Y.%m.%d.%H').date()
    meltDict[t]=meltDict.get(t,0)
    meltDict[t] += meltSumInstr#/numInstr # average over istruments
else:
  for time_ind in temp_dict:
    meltSumInstr = 0
    numInstr = 0
    for file_ind in temp_dict[time_ind]:
        #getting the temp for a given time and instrument
        T=temp_dict[time_ind][file_ind] 
        if time_ind not in volume_dict: continue
        if rcmFile[file_ind] not in volume_dict[time_ind]: continue
        #getting the vol for a given time and instrument
        V=volume_dict[time_ind][rcmFile[file_ind]] 
        melt = meltrate(V,T)
        sumMelt += melt
        meltSumInstr += melt
        numInstr += 1
    if numInstr == 0: continue
    t = dt.datetime.strptime(time_ind,'%Y.%m.%d.%H').date()
    meltDict[t]=meltDict.get(t,0)
    meltDict[t] += meltSumInstr#/numInstr # average over istruments
print 'Heat-based melt rate: ', sumMelt*measIntervalInMinutes*sec2min*cm2m/10**12
#print sumVol/2

# gets data for temperature plot
for time_ind in temp_dict:
    meltSumInstr = 0
    numInstr = 0
    for file_ind in temp_dict[time_ind]:
        T=temp_dict[time_ind][file_ind]
        if time_ind not in volume_dict: continue
        if rcmFile[file_ind] not in volume_dict[time_ind]: continue
        temp = T
        sumTemp += temp
        meltSumInstr += temp
        numInstr += 1
        sumVol += abs(V)
    if numInstr == 0: continue
    t = dt.datetime.strptime(time_ind,'%Y.%m.%d.%H').date()
    tempDict[t]=tempDict.get(t,0)
    tempDict[t] = meltSumInstr/numInstr # average over instruments

#computes meltrate based on salinity (todo)
for time_ind in sal_dict:
    meltSumInstr = 0
    numInstr = 0
    for file_ind in sal_dict[time_ind]:
        S=sal_dict[time_ind][file_ind]
        if time_ind not in volume_dict: continue
        if rcmFile[file_ind] not in volume_dict[time_ind]: continue
        V=volume_dict[time_ind][rcmFile[file_ind]] 
        sal = meltrateSalinity(V,S)
        sumSal += sal
        meltSumInstr += sal
        numInstr += 1
    if numInstr == 0: continue
    t = dt.datetime.strptime(time_ind,'%Y.%m.%d.%H').date()
    salDict[t]=salDict.get(t,0)
    salDict[t] += meltSumInstr/numInstr # average over istruments

#plotDict(tempDict)
plotMelt(meltDict,'Meltrate')
#plotMelt(meltDict,'Meltrate (heat)')
#plotMelt(salDict,'Meltrate (sal)')
     
print "Done"