import csv
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import os
import json
from scipy.spatial import Voronoi, voronoi_plot_2d
from functions import nums,haversine,calculate_bearing,polygon_area,test_all,plotVorVolumes,plotVorTemps,myImputer,myDictImputer
#from sklearn.impute import SimpleImputer
from sklearn.linear_model import LinearRegression,Lasso

#from subprocess import call
#call(["pwd"])
dirMain = os.path.realpath('..')  + "/"
if dirMain == '/Users/':
    dirMain = "/Users/christiedjidjev/OneDrive/Ice Shelf Project/" 
dir = dirMain + "Data/amisor_mooring_ascii/"
os.system("rm " + "-f " + dir + ".DS_Store")
year_col = 1
month_col = 2
day_col = 3
hour_col = 4
min_col = 5
speed_col = 6
position_col = 7
salin_col = 5
temp_col = 11
depth_col = 9
#max_column = 5
num_rows = 197357
P= np.empty((num_rows,),dtype=list) # position
#D=-1*np.ones(num_rows) #depth
#S=-1*np.ones(num_rows) # salinity
#T=-1*np.ones(num_rows) # temp
V=-1*np.ones(num_rows) # velocity
Di=-1*np.ones(num_rows) # direction
X_coord = [] # x-coord for Voronoi computation
Y_coord = [] # y-coord for Voronoi computation
point_index = {} # name for point [X_coord,Y_coord]
X_mooring = {}
Y_mooring = {} 
mooring_0 = []
mooring_1 = []
year=np.empty(num_rows, dtype=int)
month=np.empty(num_rows, dtype=int)
day=np.empty(num_rows, dtype=int)
hour=np.empty(num_rows, dtype=int)
minutes=np.empty(num_rows, dtype=int)
file_name=np.empty(num_rows,dtype=object)
volume=-np.ones(num_rows)
file_list=[]
fileName = []

amisorA = (-69.394711, 76.007650)
amisorB = (-68.429035, 70.140795)
distAB=haversine(amisorA[1], amisorA[0], amisorB[1], amisorB[0])
bearingAB = calculate_bearing(amisorB[0],amisorB[1],amisorA[0],amisorA[1])

j=0
for f in os.listdir(dir):
    print f
    with open(dir+f) as infile:
      line_num=0
      for line in infile:
          line_num= line_num+1
          if line_num ==1:
              name_val = line.split()[2]
              if (name_val == 'amisor8' or name_val == 'amisor9'): break
          if line_num ==3: 
              row= nums(line)
              #print "gps", row
              pos_val = ((row[0]+row[1]/60), row[2]+row[3]/60)
              dist=haversine(pos_val[1], -pos_val[0], amisorB[1], amisorB[0])
              point_index[f] = len(X_coord)
              fileName.append(f)
              X_coord = X_coord + [dist]
              X_mooring[name_val] = dist
              file_list += [f]              
              
          if line_num == 4:
              row=nums(line)
              #print "depth", row
              dep_val = row[0]
              Y_coord = Y_coord + [dep_val]
              if (name_val == 'amisor7'): mooring_0 += [dep_val]
              if (name_val == 'amisor1'): mooring_1 += [dep_val]
              #print name_val, dep_val
          if line_num == 6:
              row=nums(line)
              #print "depth", row
              oceanDep_val = row[0]
              Y_mooring[name_val] = oceanDep_val              
          if line_num < 14: continue
          line = " ".join(line.split()).replace(' ', '\t')
          row = line.split("\t")
          #print "j=",j,row
          #try:
              #T[j]=float(row[temp_col]) #Temperature 
          P[j]=pos_val #Position 
          #D[j]=dep_val #Depth
          #S[j]=float(row[salin_col]) #Salinity
          try:
              V[j]=float(row[speed_col]) #Velocity
          except:
            continue
          Di[j]=float(row[position_col]) #direction
          if V[j] < -990: continue
          #if Di[j] < -990: continue
          file_name[j]=f
          
          year[j]=int(row[year_col]) #Year 
          month[j]=int(row[month_col]) #Month 
          day[j]=int(row[day_col]) #Day 
          hour[j]=int(row[hour_col]) #Hour 
          minutes[j]=int(row[min_col]) #Min 
          Di[j] = Di[j] - bearingAB
          #if year[j]==2001 and month[j]==10 and day[j]==13 and hour[j]==20: print f,name_val,line
          j=j+1
          
# computation of area
X_mooring_dist = []        
for key, value in X_mooring.iteritems():
    X_mooring_dist.append([1000*value, -Y_mooring[key]-10])
    X_mooring_dist.append([1000*value, 430])
firstmooring = [[-10000, -mooring_0[i]] for i in range(len(mooring_0))]
secondmooring =  [[1000*distAB, -mooring_1[i]] for i in range(len(mooring_1))]
points = np.array([[1000*X_coord[i],-Y_coord[i]] for i in range(len(X_coord))]+ 
X_mooring_dist + firstmooring + secondmooring)
vor = Voronoi(points)
regions=[vor.regions[vor.point_region[i]] for i in range (len(X_coord))]
Vor_array = [vor.vertices[regions[i]] for i in range(len(regions))]

area = np.array([polygon_area(Vor_array[i]) for i in range(len(Vor_array))])
    

# volume and machine learning part
feature_dict = {}
temp_dict = {}


for i in range(j-1): #print X_coord[i], -Y_coord[i], area[i] 
    volume[i] = area[point_index[file_name[i]]] * np.sin(np.deg2rad(Di[i])) * V[i] 
    date_str=str(year[i])+'.'+str(month[i]).zfill(2)+'.'+str(day[i]).zfill(2)+'.'+str(hour[i])
    #date_str=str(year[i])+'.'+str(month[i])+'.'+str(day[i])+'.'+str(hour[i])
    feature_dict[date_str]=feature_dict.get(date_str,-999*np.ones(23))
    feature_dict[date_str][point_index[file_name[i]]]=volume[i]
    #temp_dict[date_str]=temp_dict.get(date_str,-999*np.ones(23))
    #temp_dict[date_str][point_index[file_name[i]]]=T[i]

#plotVorVolumes(vor,regions,feature_dict)
fname = dirMain + "Codes/temp.txt"
with open(fname, 'r') as f2:
    temp_dict = json.load(f2)
temp_dict_summer = {}          
temp_dict_winter = {}          
readFile = True

for key in temp_dict:
    l = key.split('.')
    date = l[0]+'.'+str(l[1]).zfill(2)+'.'+l[2] # format month as 2-digit
    fal2001 = '2001.03.20'
    win2001 = '2001.06.21'
    spr2001 = '2001.09.22'
    sum2001 = '2001.12.21'
    if date >= win2001 and date < spr2001:
        temp_dict_winter[key] = temp_dict[key]
    if date >= sum2001 or date < fal2001:
        temp_dict_summer[key] = temp_dict[key]
        
#plotVorTemps(vor,regions,temp_dict_summer)
#plotVorTemps(vor,regions,temp_dict_winter)

keys=feature_dict.keys()
keys.sort()
 
feature_array = []
#feature_ind2date = np.empty(len(feature_dict), dtype=object)
for key in keys:
    value = feature_dict[key]
    #feature_ind2date[len(feature_array)]=key
    feature_array += [value]
feature_array=np.asarray(feature_array)

catFileInd = {}
ind = 0
for key in temp_dict:
    for v in temp_dict[key]:
        if v not in catFileInd:
            catFileInd[v] = ind
            ind+=1

keys=temp_dict.keys()
keys.sort()

temp_array = []
for key in keys:
    value = temp_dict[key]
    val_array = -999*np.ones(21)
    for v in value:
        val_array[catFileInd[v]] = value[v]
        #print v, catFileInd[v], value[v]
    temp_array += [val_array]
temp_array=np.asarray(temp_array)

def errorCalc(feature_array):
    averages = []
    for skip in range(1,1000):
        print ".",
        sum1 = 0
        counter = 0
        for col in range(feature_array.shape[1]):
            for row in range(skip, feature_array.shape[0]):
                if feature_array[row][col] == -999 or feature_array[row-skip][col] == -999:
                    continue
                else:
                    if abs(feature_array[row][col]) > 0.5:
                        dif = abs(feature_array[row][col] - feature_array[row-skip][col])/abs(feature_array[row][col])
                        #if dif > 2: print row, col, dif
                        sum1 = sum1 + dif
                        #print sum1,dif,
                        counter += 1
        average = sum1/counter
        averages += [average]
    return averages

#average_vol_errors = errorCalc(feature_array)
fname = dirMain + "Codes/average_vol_errors.txt"
#with open(fname, 'w') as f:
#    json.dump(average_vol_errors,f)

#average_temp_errors = errorCalc(temp_array)
#when computing
#fname = dirMain + "Codes/average_temp_errors.txt"
#with open(fname, 'w') as f:
#    json.dump(average_temp_errors,f)
#when plotting
with open(fname, 'r') as f:
    average_vol_errors = json.load(f)
plt.plot(average_vol_errors)
plt.xlabel('Skip distance')
plt.ylabel('Average Error')
plt.grid()
plt.show()

#print feature_array[0]
imp = SimpleImputer(missing_values=-999, strategy='mean') #defines the way the data is used
feature_array=imp.fit_transform(feature_array)
#feature_array = myImputer(feature_array)

tw=120 #time_window length
feature_array_aver=np.zeros([feature_array.shape[0]-tw+2, feature_array.shape[1]])
coeff=14**3
for col in range(feature_array_aver.shape[1]):
    feature_array_aver[1:,col]=np.convolve(feature_array[:,col], np.ones((tw,))/tw, mode='valid')
    feature_array_aver[0,col]=area[col]*coeff

y=np.zeros(feature_array_aver.shape[0])
y[0]=np.sum(area)*coeff
#reg = LinearRegression()
reg=Lasso(alpha=0.0001,precompute=True,max_iter=10000,positive=True, random_state=9999, selection='random')
reg.fit(feature_array_aver, y)
print np.sum(area), np.sum(np.multiply(area,reg.coef_))

#area=np.multiply(area,reg.coef_)

for i in range(j-1): #print X_coord[i], -Y_coord[i], area[i] 
    volume[i] = area[point_index[file_name[i]]] * np.sin(np.deg2rad(Di[i])) * V[i] 
test_all(j,volume,Di)

#compute volumes
volume_dict = {}
for i in range(j-1): #print X_coord[i], -Y_coord[i], area[i] 
    date_str=str(year[i])+'.'+str(month[i]).zfill(2)+'.'+str(day[i]).zfill(2)+'.'+str(hour[i])
    #date_str=str(year[i])+'.'+str(month[i])+'.'+str(day[i]).zfill(2)+'.'+str(hour[i])
    volume_dict[date_str]=volume_dict.get(date_str,{})
    volume_dict[date_str][file_name[i]]=volume[i]

volume_dict = myDictImputer(volume_dict)

# write computed volumes to file
fname = dirMain + "Codes/vol.txt"
with open(fname, 'w') as f:
    json.dump(volume_dict,f)

# computing volumes with modified areas
for i in range(j-1): #print X_coord[i], -Y_coord[i], area[i] 
    volume[i] = area[point_index[file_name[i]]] * np.sin(np.deg2rad(Di[i])) * V[i] 
test_all(j,volume,Di)

#test()
print "Done"
